/********************************************************
 * Data Visualization - 2D Matrix Demo                  *
 * Tested in Processing 3                               *     
 *                                                      *
 * Author: Rodger (Jieliang) Luo                        *
 * Supervisor: George Legrady                           *
 *                                                      *
 * Purpose: Demonstrate how to make a simple 2D Matrix  *
 *          data visualization with the following       *
 *          features: normalization, reorder, and       *
 *          applying color image.                       *
 *                                                      *
 * Usage:   Press 'n' to switch between normalization   *
 *          and regular data value.                     *
 *          Press 'r' to switch between reorder matrix  *
 *          and regular matrix.                         *
 *          Press '1', '2', '3', '4' to apply different *
 *          color schemes.                              *
 *          Press '0' to use gray scale.                *
 ********************************************************/

/*
Step One: Upload CSV to Table & Get basic information of data. 
Step Two: Write data from Table to 2D Array & Have max and min value of data.
Step Three: Visualize 2D Matrix.
Step Four: Draw labels and title.
*/
Table table1, table2, table3;
float[][] dataMatrix1, dataMatrix2, dataMatrix3;
int numRows;
int numCols;
int startyear = 2006;
float cellwidth;
float cellheight;
float maxValue1, maxValue2, maxValue3;
float minValue1, minValue2, minValue3;
float[] maxes1, maxes2, maxes3;

float constantVerticalMargin = 100;
float constantHorizontalMargin = 160;

//interaction
boolean normalisation = false;
boolean colorApply = false;
boolean reorder = false;



//Only run one time
void setup() {
  size(1280, 720); // setup the size of the window
  // In Processing 3, the size() function must be the first line of code inside setup()
  // and needs to use numbers instead of variables. 

  table1 = loadTable("order_of_the_phoenix.csv", "header");
  table2 = loadTable("half_blood_prince.csv", "header");
  table3 = loadTable("deathly_hallows.csv", "header");
  numRows = table1.getRowCount();
  numCols = table1.getColumnCount();
  dataMatrix1 = new float[numCols-1][numRows];
  dataMatrix2 = new float[numCols-1][numRows];
  dataMatrix3 = new float[numCols-1][numRows];
  
  for (int i=0 ; i<numCols-1 ; i++) {
    for (int j=0 ; j<numRows ; j++) {
      dataMatrix1[i][j] = table1.getFloat(j,i+1);
    }
  }
  
  for (int i=0 ; i<numCols-1 ; i++) {
    for (int j=0 ; j<numRows ; j++) {
      dataMatrix2[i][j] = table2.getFloat(j,i+1);
    }
  }
  
  for (int i=0 ; i<numCols-1 ; i++) {
    for (int j=0 ; j<numRows ; j++) {
      dataMatrix1[i][j] = table1.getFloat(j,i+1);
    }
  }
  
  maxValue1 = max2D(dataMatrix1);
  minValue1 = min2D(dataMatrix1);
  maxValue2 = max2D(dataMatrix2);
  minValue2 = min2D(dataMatrix2);
  maxValue3 = max2D(dataMatrix3);
  minValue3 = min2D(dataMatrix3);

  PFont f;
  f = createFont("Arial", 24);
  textFont(f);

  //println("Max Checkouts : ",maxValue," Min Checkouts : ",minValue);
  
}
//Refresh the canvas every frame
void draw() {  
  background(250);
  surface.setResizable(true);
  
  cellwidth = (width - 2*constantHorizontalMargin)/(numRows);
  cellheight = (height - 3*constantVerticalMargin)/(3*(numCols-1));
  
  for (int i=0 ; i<numCols-1 ; i++) {
    for (int j=0 ; j<numRows ; j++) {
      stroke(255);
      strokeWeight(1);
      float grayscale;
      grayscale = map(dataMatrix1[i][j],maxValue1,minValue1,0,255);
      fill(grayscale);
      rect(
      constantHorizontalMargin + cellwidth*j,
      constantVerticalMargin + cellheight*(i),
      cellwidth,
      cellheight); 
    }
    for (int j=0 ; j<numRows ; j++) {
      stroke(255);
      strokeWeight(1);
      float grayscale;
      grayscale = map(dataMatrix2[i][j],maxValue2,minValue2,0,255);
      fill(grayscale);
      rect(
      constantHorizontalMargin + cellwidth*j,
      1.25*constantVerticalMargin + cellheight*(i+numCols),
      cellwidth,
      cellheight);
    }
    for (int j=0 ; j<numRows ; j++) {
      stroke(255);
      strokeWeight(1);
      float grayscale;
      grayscale = map(dataMatrix3[i][j],maxValue3,minValue3,0,255);
      fill(grayscale);
      rect(
      constantHorizontalMargin + cellwidth*j,
      1.5*constantVerticalMargin + cellheight*(i+2*numCols),
      cellwidth,
      cellheight);
    }
  }

  for (int i=0 ; i<numRows/12 ; i++)
  {
    textAlign(RIGHT,CENTER);
    textSize(10);
    fill(0);
    text(startyear + i,constantHorizontalMargin+cellwidth*(i*12 + 6),constantVerticalMargin*1.5+cellheight*3*numCols);
  }
  textAlign(RIGHT,CENTER);
  textSize(14);
  fill(0);
  text("books",constantHorizontalMargin,constantVerticalMargin+cellheight*0+cellheight/2);
  text("dvds",constantHorizontalMargin,constantVerticalMargin+cellheight*1+cellheight/2);
  text("audiobooks",constantHorizontalMargin,constantVerticalMargin+cellheight*2+cellheight/2);
  text("cds",constantHorizontalMargin,constantVerticalMargin+cellheight*3+cellheight/2);
  
  text("books",constantHorizontalMargin,1.25*constantVerticalMargin+cellheight*(0+numCols)+cellheight/2);
  text("dvds",constantHorizontalMargin,1.25*constantVerticalMargin+cellheight*(1+numCols)+cellheight/2);
  text("audiobooks",constantHorizontalMargin,1.25*constantVerticalMargin+cellheight*(2+numCols)+cellheight/2);
  text("cds",constantHorizontalMargin,1.25*constantVerticalMargin+cellheight*(3+numCols)+cellheight/2);
  
  text("books",constantHorizontalMargin,1.5*constantVerticalMargin+cellheight*(0+2*numCols)+cellheight/2);
  text("dvds",constantHorizontalMargin,1.5*constantVerticalMargin+cellheight*(1+2*numCols)+cellheight/2);
  text("audiobooks",constantHorizontalMargin,1.5*constantVerticalMargin+cellheight*(2+2*numCols)+cellheight/2);
  text("cds",constantHorizontalMargin,1.5*constantVerticalMargin+cellheight*(3+2*numCols)+cellheight/2);
  
  textSize(24);
  textAlign(CENTER, CENTER);
  text("Checkout frequency per month in Harry Potter media",width/2, constantVerticalMargin/2);
  textSize(18);
  text("Order of the Phoenix",width/2, constantVerticalMargin*0.85);
  text("Half-Blood Prince",width/2, 1.45*constantVerticalMargin+cellheight*(numCols-1));
  text("Deathly Hallows",width/2, 1.7*constantVerticalMargin+cellheight*(2*numCols-1));
  //println("Normalisation : "+normalisation+"; ColorApply : "+colorApply+"; Reorder : "+reorder);


  
  if(normalisation == true) normalise();
  if(normalisation == false) reset_n();
  textSize(12);
  text("Press 'n' to\n normalise/denormalize",constantHorizontalMargin+cellwidth*numRows+60,constantVerticalMargin*1.55+cellheight*(numCols+2)-15);
  
  
  /*if(reorder == true) resort();
  if(reorder == false) reset_r();*/
  
  if(colorApply == true) ColorScheme();
  
  fill(0);
  text("Press '1,2,3,4' to\n pick color scheme",constantHorizontalMargin+cellwidth*numRows+60,constantVerticalMargin*1.55+cellheight*(numCols+3)-15);

  textSize(10);

  stroke(0);
  strokeWeight(3);
  fill(0);
  triangle(constantHorizontalMargin+cellwidth*numRows+20,constantVerticalMargin+cellheight*(2)-20,constantHorizontalMargin+cellwidth*numRows+15,constantVerticalMargin+cellheight*(2)-5,constantHorizontalMargin+cellwidth*numRows+25,constantVerticalMargin+cellheight*(2)-5);
  text("theatrical\nrelease",constantHorizontalMargin+cellwidth*numRows+50,constantVerticalMargin+cellheight*2-15);

  stroke(0);
  strokeWeight(3);
  fill(255);
  triangle(constantHorizontalMargin+cellwidth*numRows+20,constantVerticalMargin+cellheight*(3)-20,constantHorizontalMargin+cellwidth*numRows+15,constantVerticalMargin+cellheight*(3)-5,constantHorizontalMargin+cellwidth*numRows+25,constantVerticalMargin+cellheight*(3)-5);
  fill(0);
  text("DVD\nrelease",constantHorizontalMargin+cellwidth*numRows+50,constantVerticalMargin+cellheight*3-15);


  stroke(0);
  //text("theatrical\nrelease",constantHorizontalMargin+cellwidth*18-25,constantVerticalMargin*1.12+cellheight*(numCols-1));
  strokeWeight(3);
  fill(0);
  triangle(constantHorizontalMargin+cellwidth*18,constantVerticalMargin+cellheight*(numCols-1)+5,constantHorizontalMargin+cellwidth*18-5,constantVerticalMargin*1.25+cellheight*(numCols-1)-5,constantHorizontalMargin+cellwidth*18+5,constantVerticalMargin*1.25+cellheight*(numCols-1)-5);
  
  stroke(0);
  strokeWeight(3);
  fill(255);
  triangle(constantHorizontalMargin+cellwidth*22,constantVerticalMargin+cellheight*(numCols-1)+5,constantHorizontalMargin+cellwidth*22-5,constantVerticalMargin*1.25+cellheight*(numCols-1)-5,constantHorizontalMargin+cellwidth*22+5,constantVerticalMargin*1.25+cellheight*(numCols-1)-5);
  //text("dvd\nrelease",constantHorizontalMargin+cellwidth*22+20,constantVerticalMargin*1.12+cellheight*(numCols-1));
  
  stroke(0);
  strokeWeight(3);
  fill(0);
  triangle(constantHorizontalMargin+cellwidth*42,constantVerticalMargin*1.5+cellheight*(2*numCols-1)-20,constantHorizontalMargin+cellwidth*42-5,constantVerticalMargin*1.55+cellheight*(2*numCols-1)-10,constantHorizontalMargin+cellwidth*42+5,constantVerticalMargin*1.55+cellheight*(2*numCols-1)-10);
  
  stroke(0);
  strokeWeight(3);
  fill(255);
  triangle(constantHorizontalMargin+cellwidth*47,constantVerticalMargin*1.5+cellheight*(2*numCols-1)-20,constantHorizontalMargin+cellwidth*47-5,constantVerticalMargin*1.55+cellheight*(2*numCols-1)-10,constantHorizontalMargin+cellwidth*47+5,constantVerticalMargin*1.55+cellheight*(2*numCols-1)-10);
  
  stroke(0);
  strokeWeight(3);
  fill(0);
  triangle(constantHorizontalMargin+cellwidth*58,constantVerticalMargin*1.5+cellheight*(3*numCols-1)+8,constantHorizontalMargin+cellwidth*58-5,constantVerticalMargin*1.75+cellheight*(3*numCols-1),constantHorizontalMargin+cellwidth*58+5,constantVerticalMargin*1.75+cellheight*(3*numCols-1));
  fill(255);
  strokeWeight(3);
  triangle(constantHorizontalMargin+cellwidth*63,constantVerticalMargin*1.5+cellheight*(3*numCols-1)+8,constantHorizontalMargin+cellwidth*63-5,constantVerticalMargin*1.75+cellheight*(3*numCols-1),constantHorizontalMargin+cellwidth*63+5,constantVerticalMargin*1.75+cellheight*(3*numCols-1));
  
  stroke(0);
  strokeWeight(3);
  fill(0);
  triangle(constantHorizontalMargin+cellwidth*66,constantVerticalMargin*1.5+cellheight*(3*numCols-1)+8,constantHorizontalMargin+cellwidth*66-5,constantVerticalMargin*1.75+cellheight*(3*numCols-1),constantHorizontalMargin+cellwidth*66+5,constantVerticalMargin*1.75+cellheight*(3*numCols-1));
  fill(255);
  strokeWeight(3);
  triangle(constantHorizontalMargin+cellwidth*70,constantVerticalMargin*1.5+cellheight*(3*numCols-1)+8,constantHorizontalMargin+cellwidth*70-5,constantVerticalMargin*1.75+cellheight*(3*numCols-1),constantHorizontalMargin+cellwidth*70+5,constantVerticalMargin*1.75+cellheight*(3*numCols-1));
}